//***************************************************************************************************
// file mode_rdma.cpp
#include <libce.h>

const ioaddr iBR = 0x01F0;
const ioaddr iERR = 0x01F1;
const ioaddr iSCR = 0x01F2;
const ioaddr iSNR = 0x01F3;
const ioaddr iCNL = 0x01F4;
const ioaddr iCNH = 0x01F5;
const ioaddr iHND = 0x01F6;
const ioaddr iCMD = 0x01F7;
const ioaddr iSTS = 0x01F7;
const ioaddr iDCR = 0x03F6;
const ioaddr iASR = 0x03F6;

ioaddr iBMCMD;	
ioaddr iBMSTR;
ioaddr iBMDTPR;

const natl sincr= 12;
natl prd[2];

void cerca_bm(natb& dev, natb& fun)
{	for (dev = 0; dev < 32; dev++)
		for (fun = 0; fun < 8; fun++)
			if (pci_read_confw(0, dev, fun, 10) == 0x0101)
				return; 
}

void init_bm() 
{	natl base;  natw cmd; natb bus = 0, dev, fun;
	cerca_bm(dev, fun);
	base = pci_read_confl(bus, dev, fun, 32); 		// registro base 4, blocco di I/O
	base &= 0xFFFFFFFE; 						// bit n. 0 posto a 0 (valeva 1)
	iBMCMD = base;
	iBMSTR = base + 2;
	iBMDTPR = base + 4;
	cmd = pci_read_confw(bus, dev, fun, 4); 		// lettura del Command Register
	pci_write_confw(bus, dev, fun, 4, cmd | 0x0005); // abilit. (bit n. 0 e n. 2 posti a 1)
}
extern "C" void a_go_rdma();
extern "C" void a_driver_rdma();
void ini()
{	init_bm();
	gate_init(252, &a_go_rdma);
	gate_init(52, &a_driver_rdma);
	apic_set_VECT(14, 52);
	apic_set_MIRQ(14, false);		// abilitazione piedino IO APIC
}

extern "C" void c_go_rdma(natl indse, natb nn, natb vv[])
{	natb work; natq ww;

	// comandi per il bus master PCI (ponte ATA-PCI) 
	ww = reinterpret_cast<natq>(&vv[0]);				// scrittura in prd
	prd[0] = static_cast<natl>(ww);				// scrittura in prd
	prd[1] = 0x80000000 | nn*512;
	ww = reinterpret_cast<natq>(&prd[0]);				// scrittura in prd
	outputl(static_cast<natl>(ww), iBMDTPR); 	// scrittura in BMDTPR
	inputb(iBMCMD, work);
	work |= 0x08;					// bit n. 3 di BMCMD posto a 1 (lettura da PCI) 
	outputb(work, iBMCMD);
	inputb(iBMSTR, work);
	work &= 0xF9;					// bit n. 2-1 di BMSTR posto a 0 (Interr. ed Errore)
	outputb(work, iBMSTR);

	// comandi per il controllore ATA
	natb lba_0 = indse,
		lba_1 = indse >> 8,
		lba_2 = indse >> 16,
		lba_3 = indse >> 24;
	outputb(lba_0, iSNR);			// indirizzo settore e selezione dispositivo
	outputb(lba_1, iCNL);
	outputb(lba_2, iCNH);
	natb hnd = (lba_3 & 0x0F) | 0xE0; // bit n. 0-3 indirizzo, bit n. 4 posto a 0 (master)
	outputb(hnd, iHND); 				// bit n. 6 (e altri bit non significativi) posti a 1
	outputb(nn, iSCR);				// numero di settori
	outputb(0x00, iDCR);			// // abilitazione interruzioni (bit n. 1 posti a 0)
	outputb(0xC8, iCMD);			// comando di lettura in DMA

	// avvio dell'operazione
	inputb(iBMCMD, work);
	work |= 0x01;					// porta a 1 il bit n. 0 (Start/stop)
	outputb(work, iBMCMD);

	sem_wait(sincr);
}

extern "C" void c_driver_rdma()
{	natb work;
	inputb(iBMCMD, work);
	work &= 0xFE;					// porta a 0 il bit n. 0 (Start/stop)
	outputb(work, iBMCMD);
	inputb(iBMSTR, work);			// risposta alla richiesta di interruzione
	inputb(iSTS, work);
	outputb(0x0A,iDCR); 			// disabilitazione interruzioni

	sem_signal(sincr);
}
//***************************************************************************************************
